#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>
#include <PubSubClient.h>

// Update these with your WiFi credentials
const char* ssid = WIFI_SSID;
const char* password = WIFI_PASS;

// Update these with your MQTT Broker details
const char* mqtt_server = MQTT_SERVER;
const unsigned int mqtt_port = MQTT_PORT;
const char* mqtt_user = MQTT_USER;
const char* mqtt_pass = MQTT_PASS;

// Certificate
const char* ca_cert = CA_CERT

const char* mqtt_topic = "backdoor/control";

WiFiClientSecure espClient;
PubSubClient client(espClient);

void setup_wifi() {
    delay(10);
    // Connecting to a WiFi network
    WiFi.begin(ssid, password);
    while (WiFi.status() != WL_CONNECTED) {
        delay(500);
        Serial.print(".");
    }
    Serial.println("");
    Serial.println("WiFi connected");
    Serial.println("IP address: ");
    Serial.println(WiFi.localIP());

    // Load certificate
    X509List cert(ca_cert);
    espClient.setTrustAnchors(&cert);
}

void callback(char* topic, byte* payload, unsigned int length) {
    payload[length] = '\0'; // Null-terminate the payload
    String message = String((char*)payload);
    Serial.print("Message arrived [");
    Serial.print(topic);
    Serial.print("] ");
    Serial.println(message);

    if (String(topic) == mqtt_topic) {
        if (message == "unlock") {
            digitalWrite(D3, LOW);
            delay(500); // 1/2 second
            digitalWrite(D3, HIGH);
        } else if (message == "lock") {
            digitalWrite(D4, LOW);
            delay(500); // 1/2 second
            digitalWrite(D4, HIGH);
        }
    }
}

void reconnect() {
    // Loop until we're reconnected
    while (!client.connected()) {
        Serial.print("Attempting MQTT connection...");
        // Attempt to connect
        if (client.connect("ESP8266Client", mqtt_user, mqtt_pass)) {
            Serial.println("connected");
            // Subscribe
            client.subscribe(mqtt_topic);
        } else {
            Serial.print("failed, rc=");
            Serial.print(client.state());
            Serial.println(" try again in 5 seconds");
            // Wait 5 seconds before retrying
            delay(5000);
        }
    }
}

void setup() {
    Serial.begin(115200);
    delay(10);
    Serial.println("Starting setup");
    setup_wifi();
    client.setServer(mqtt_server, mqtt_port);
    client.setCallback(callback);

    pinMode(D3, OUTPUT);
    pinMode(D4, OUTPUT);
    digitalWrite(D3, HIGH);
    digitalWrite(D4, HIGH);
}

void loop() {
    if (!client.connected()) {
        reconnect();
    }
    client.loop();
}